/*
--------------------------------------------------------------------------------

Author: Enrico Gullotti
Website: http://www.illusioncatalyst.com

Name: IC.SizeInScreen
Category: IllusionCatalyst Tools
Version: 0.2 - 2009.03.04

SizeInScreen calculates the apparent dimension measured in pixels of the current
selection in Screen Coordinate System. Works on Editable Polys and Meshes.

Get Size retrieves current selection dimensions in screen

X and Y fields provide feedback and can be used to input dimensions to adjust
current view.

Update on selection change sets the script to provide feedback on every
selection change.

Update on Point Of View change sets the script to provide feedback while the
point of view is changed by rotation, pan and zoom.

Frequency Ratio applies to Update on Point Of View Change. Higher values
correspond to more frequents updates. The overall frequency scales taking into
account the complexity of the current selection.

Preset Storage offers four positions to fill with dimensions that can be set
to >X or >Y.

--------------------------------------------------------------------------------
*/



--macroScript IC_SizeInScreen
----category:"IllusionCatalyst Tools"
--buttonText:"SizeInScreen"
--tooltip:"IC.SizeInScreen"
(
	global rol_IC_SizeInScreen

	rollout rol_IC_SizeInScreen "IC.SizeInScreen 0.2"
	(
		local aoSelection = undefined
		local iNumAllVerts = 0
		local fScaleFactor = 50
		local bStopGap = false

		timer tStopGap interval:500 active:false

		local iMaxVersion = (maxVersion())[1] / 1000.0

--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--
-- INTERFACE
--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--


		button btGetSize "Get Size" width:45 height:17 align:#left offset:[-4, 4] across:5
		label lbX "X:" align:#left offset:[20, 6]
		editText etWidth text:"0" fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[1, 4]
		label lbY "Y:" align:#left offset:[14, 6]
		editText etHeight text:"0" fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[-5, 4]

		groupBox gbxSize width:148 height:33 align:#left offset:[-9, -33]

		checkButton cbtOnSelChange "Selection" width:69 height:16 align:#left offset:[-4, 16] across:2
		checkButton cbtOnViewChange "Point of View" width:69 height:16 align:#left offset:[0, 16]

		label lbUpdateFreq "Frequency Ratio" align:#left offset:[0, -1] across:2
		spinner spUpadateFreq range:[1, 10, 5] type:#integer fieldWidth:24 align:#left offset:[34, -2]
		groupBox gbxUpdateFreq width:104 height:22 align:#left offset:[-4, -27]

		groupBox gbxUpdate " Update on change " width:148 height:56 align:#left offset:[-9, -56]

		editText etStorage01 fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[-8, 16] across:6
		button btSetXTo01 ">X" width:18 height:17 align:#left offset:[6, 16]
		button btSetYTo01 ">Y" width:18 height:17 align:#left offset:[3, 16]

		editText etStorage02 fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[0, 16]
		button btSetXTo02 ">X" width:18 height:17 align:#left offset:[14, 16]
		button btSetYTo02 ">Y" width:18 height:17 align:#left offset:[11, 16]

		editText etStorage03 fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[-8, -2] across:6
		button btSetXTo03 ">X" width:18 height:17 align:#left offset:[6, -2]
		button btSetYTo03 ">Y" width:18 height:17 align:#left offset:[3, -2]

		editText etStorage04 fieldWidth:31 bold:false labelOnTop:false readOnly:false align:#left offset:[0, -2]
		button btSetXTo04 ">X" width:18 height:17 align:#left offset:[14, -2]
		button btSetYTo04 ">Y" width:18 height:17 align:#left offset:[11, -2]

		groupBox gbxStorage " Presets Storage " width:148 height:58 align:#left offset:[-9, -58]


--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--
-- FUNCTIONS
--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--


		function getSelObjects =
		(
			aoSelection = undefined
			iNumAllVerts = 0

			if (selection.count == 0) then
			(
				aoSelection = undefined
				iNumAllVerts = fScaleFactor * 1000
			)
			else
			(
				for oItem in selection do
				(
					if ( ((classOf oItem) != Editable_Poly) and ((classOf oItem) != Editable_Mesh) ) then
					(
						aoSelection = undefined
						iNumAllVerts = fScaleFactor * 1000
						exit
					)
					else
					(
						iNumAllVerts += oItem.numVerts
						aoSelection = selection
					)
				)
			)

			tStopGap.interval = iNumAllVerts / fScaleFactor
		)

---------------------------------------

		function getSizeOnViewChange =
		(
			if (aoSelection != undefined) then
			(
				if (bStopGap == false) then
				(
					bStopGap = true
					tStopGap.active = true

					gw.setTransform (Matrix3 1)

					local iNumVerts = 0
					local ap3VertPos = #()

					local aiVertPosX = #()
					local aiVertPosY = #()

					local p3TransPoint = [0,0,0]

					for oItem in aoSelection do
					(
						ap3VertPos = #()
						iNumVerts = oItem.numVerts

						if ((classOf oItem) == Editable_Poly) then
							ap3VertPos = for i = 1 to iNumVerts collect (polyOp.getVert oItem i)
						else if ((classOf oItem) == Editable_Mesh) then
							ap3VertPos = for i = 1 to iNumVerts collect (meshOp.getVert oItem i)

						for p3VertPos in ap3VertPos do
						(
							p3TransPoint = gw.transPoint p3VertPos
							append aiVertPosX (ceil(p3TransPoint).x)
							append aiVertPosY (ceil(p3TransPoint).y)
						)
					)

					etWidth.text = (((aMax aiVertPosX - aMin aiVertPosX) as Integer) as String)
					etHeight.text = (((aMax aiVertPosY - aMin aiVertPosY) as Integer) as String)
				)
			)
			else
			(
				etWidth.text = "0"
				etHeight.text = "0"
			)
		)

---------------------------------------

		function getSizeOnSelChange =
		(
			if (aoSelection != undefined) then
			(
				gw.setTransform (Matrix3 1)

				local iNumVerts = 0
				local ap3VertPos = #()

				local aiVertPosX = #()
				local aiVertPosY = #()

				local p3TransPoint = [0,0,0]

				for oItem in aoSelection do
				(
					ap3VertPos = #()
					iNumVerts = oItem.numVerts

					if ((classOf oItem) == Editable_Poly) then
						ap3VertPos = for i = 1 to iNumVerts collect (polyOp.getVert oItem i)
					else if ((classOf oItem) == Editable_Mesh) then
						ap3VertPos = for i = 1 to iNumVerts collect (meshOp.getVert oItem i)

					for p3VertPos in ap3VertPos do
					(
						p3TransPoint = gw.transPoint p3VertPos
						append aiVertPosX (ceil(p3TransPoint).x)
						append aiVertPosY (ceil(p3TransPoint).y)
					)
				)

				etWidth.text = (((aMax aiVertPosX - aMin aiVertPosX) as Integer) as String)
				etHeight.text = (((aMax aiVertPosY - aMin aiVertPosY) as Integer) as String)
			)
			else
			(
				etWidth.text = "0"
				etHeight.text = "0"
			)
		)

---------------------------------------

		function getSizeInScreen =
		(
			local ap2Result = [0,0]

			if (aoSelection != undefined) then
			(
				gw.setTransform (Matrix3 1)

				local iNumVerts = 0
				local ap3VertPos = #()

				local aiVertPosX = #()
				local aiVertPosY = #()

				local p3TransPoint = [0,0,0]

				for oItem in aoSelection do
				(
					ap3VertPos = #()
					iNumVerts = oItem.numVerts

					if ((classOf oItem) == Editable_Poly) then
						ap3VertPos = for i = 1 to iNumVerts collect (polyOp.getVert oItem i)
					else if ((classOf oItem) == Editable_Mesh) then
						ap3VertPos = for i = 1 to iNumVerts collect (meshOp.getVert oItem i)

					for p3VertPos in ap3VertPos do
					(
						p3TransPoint = gw.transPoint p3VertPos
						append aiVertPosX p3TransPoint.x
						append aiVertPosY p3TransPoint.y
					)
				)

				ap2Result = [(aMax aiVertPosX - aMin aiVertPosX), (aMax aiVertPosY - aMin aiVertPosY)]
			)
			return ap2Result
		)

--------------------------------------------------------------------------------

	function setSize nAxis iSize =
	(
		undo on
		(
			if (iMaxVersion < 10) then
			(
				if (aoSelection != undefined) then
				(
					local p3ViewCameraPos = (inverse(getViewTM())).row4
					local p3ViewCameraDir = -(inverse(getViewTM())).row3

					local p3SelCenter = aoSelection.center
					local fTestCamDist = length(aoSelection.max - aoSelection.min)

					local camDummy = freeCamera wireColor:(color 102 102 102)

					if (nAxis == #x) then
						camDummy.fovType = 1
					else if (nAxis == #y) then
						camDummy.fovType = 2
					else
					(
						delete camDummy
						throw "Wrong input in function setSize()"
					)

					camDummy.curFov = 50.0

					if (viewport.IsPerspView() == false) then
						camDummy.orthoProjection = true

					camDummy.transform = inverse(getViewTm())

					if (viewport.IsPerspView() == false) then
						camDummy.position = p3SelCenter - p3ViewCameraDir *fTestCamDist
					else
					(
						local p3CamSelDir = normalize(p3SelCenter - p3ViewCameraPos)
						local fMinCamSelDist = length(p3ViewCameraPos - p3SelCenter) * (dot p3ViewCameraDir p3CamSelDir)
						camDummy.position = p3SelCenter - p3ViewCameraDir * fMinCamSelDist
					)

					viewport.setCamera camDummy
					completeRedraw()

					local p2TestSize = getSizeInScreen()

					if (nAxis == #x) then
						local fRatio = iSize / p2TestSize.x
					else if (nAxis == #y) then
						local fRatio = iSize / p2TestSize.y

					local fNewFov = (atan(tan(camDummy.curFov / 2) / fRatio)) * 2

					camDummy.curFov = fNewFov
					completeRedraw()

					if (viewport.IsPerspView() == false) then
						viewport.setType #view_iso_user
					else
						viewport.setType #view_persp_user

					delete camDummy
				)
			)
			else
			(
				if (aoSelection != undefined) then
				(
					if (viewport.IsPerspView() == true) then
					(
						local p3ViewCameraPos = (inverse(getViewTM())).row4
						local p3ViewCameraDir = -(inverse(getViewTM())).row3

						local p3SelCenter = aoSelection.center
						local fTestCamDist = length(aoSelection.max - aoSelection.min)

						local camDummy = freeCamera wireColor:(color 102 102 102)

						if (nAxis == #x) then
							camDummy.fovType = 1
						else if (nAxis == #y) then
							camDummy.fovType = 2
						else
						(
							delete camDummy
							throw "Wrong input in function setSize()"
						)

						camDummy.curFov = 50.0
						camDummy.orthoProjection = false
						camDummy.transform = inverse(getViewTm())

						local p3CamSelDir = normalize(p3SelCenter - p3ViewCameraPos)
						local fMinCamSelDist = length(p3ViewCameraPos - p3SelCenter) * (dot p3ViewCameraDir p3CamSelDir)
						camDummy.position = p3SelCenter - p3ViewCameraDir * fMinCamSelDist

						viewport.setCamera camDummy
						completeRedraw()

						local p2TestSize = getSizeInScreen()

						if (nAxis == #x) then
							local fRatio = iSize / p2TestSize.x
						else if (nAxis == #y) then
							local fRatio = iSize / p2TestSize.y

						local fNewFov = (atan(tan(camDummy.curFov / 2) / fRatio)) * 2

						camDummy.curFov = fNewFov
						completeRedraw()

						viewport.setType #view_persp_user

						delete camDummy
					)
					else
					(
						with redraw off
						(
							max zoomext sel

							local p2TestSize = getSizeInScreen()

							if (nAxis == #x) then
								local fRatio = iSize / p2TestSize.x
							else if (nAxis == #y) then
								local fRatio = iSize / p2TestSize.y
							else
								throw "Wrong input in function setSize()"

							viewport.zoom (1 / fRatio)
						)
						completeRedraw()
					)
				)
			)
		)
	)


--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--
-- EVENTS
--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx--


		on tStopGap tick do
		(
			bStopGap = false
			tStopGap.active = false
		)

---------------------------------------

		on btGetSize pressed do
		(
			getSelObjects()
			getSizeOnSelChange()
		)

		on etWidth entered arg do
		(
			local iValue = arg as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #x iValue
				getSizeOnSelChange()
			)
		)

		on etHeight entered arg do
		(
			local iValue = arg as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #y iValue
				getSizeOnSelChange()
			)
		)

---------------------------------------

		on cbtOnSelChange changed bState do
		(
			if (bState == true) then
			(
				getSelObjects()
				getSizeOnSelChange()

				callbacks.addScript #selectionSetChanged "rol_IC_SizeInScreen.getSizeOnSelChange()" id:#cb_IC_SizeOnSelChange
			)
			else
			(
				callbacks.removeScripts id:#cb_IC_SizeOnSelChange
			)
		)

		on cbtOnViewChange changed bState do
		(
			if (bState == true) then
			(
				getSelObjects()
				getSizeOnSelChange()

				callbacks.addScript #viewportChange "rol_IC_SizeInScreen.getSizeOnViewChange()" id:#cb_IC_SizeOnViewChange
			)
			else
			(
				callbacks.removeScripts id:#cb_IC_SizeOnViewChange
			)
		)

		on spUpadateFreq changed iValue do
		(
			fScaleFactor = iValue * 10
			tStopGap.interval = iNumAllVerts / fScaleFactor
		)

---------------------------------------

		on btSetXTo01 pressed do
		(
			local iValue = etStorage01.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #x iValue
				getSizeOnSelChange()
			)
		)

		on btSetYTo01 pressed do
		(
			local iValue = etStorage01.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #y iValue
				getSizeOnSelChange()
			)
		)

---------------------------------------

		on btSetXTo02 pressed do
		(
			local iValue = etStorage02.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #x iValue
				getSizeOnSelChange()
			)
		)

		on btSetYTo02 pressed do
		(
			local iValue = etStorage02.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #y iValue
				getSizeOnSelChange()
			)
		)

---------------------------------------

		on btSetXTo03 pressed do
		(
			local iValue = etStorage03.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #x iValue
				getSizeOnSelChange()
			)
		)

		on btSetYTo03 pressed do
		(
			local iValue = etStorage03.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #y iValue
				getSizeOnSelChange()
			)
		)

---------------------------------------

		on btSetXTo04 pressed do
		(
			local iValue = etStorage04.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #x iValue
				getSizeOnSelChange()
			)
		)

		on btSetYTo04 pressed do
		(
			local iValue = etStorage04.text as Integer
			if ( (iValue != undefined) and (iValue != 0) ) then
			(
				setSize #y iValue
				getSizeOnSelChange()
			)
		)

--------------------------------------------------------------------------------

		on rol_IC_SizeInScreen open do
		(
			getSelObjects()
			getSizeOnSelChange()

			callbacks.addScript #selectionSetChanged "rol_IC_SizeInScreen.getSelObjects()" id:#cb_IC_SizeGetSel
		)

		on rol_IC_SizeInScreen close do
		(
			callbacks.removeScripts id:#cb_IC_SizeGetSel
			callbacks.removeScripts id:#cb_IC_SizeOnSelChange
			callbacks.removeScripts id:#cb_IC_SizeOnViewChange
		)

--------------------------------------------------------------------------------

	) -- End Rollout

	on execute do
	(
		try
		(
			destroyDialog rol_IC_SizeInScreen
		)
		catch ()

		createDialog rol_IC_SizeInScreen 156 159 2 87 style:#(#style_toolwindow, #style_border, #style_sysmenu) lockHeight:true lockWidth:true
	)

) -- End MacroScript